@@ -25,11 +25,6 @@
  * SUCH DAMAGE.
  */
 
-#ifdef __APPLE__
-#  include <mach/mach_time.h>
-#  include <mach/kern_return.h>
-#endif
-
 #include <sys/param.h>
 #include <sys/time.h>
 
@@ -58,87 +53,6 @@
 #  define _PATH_DEVNULL "/dev/null"
 #endif
 
-const char *
-get_hostname(char *buf, size_t buflen, int short_hostname)
-{
-	char *p;
-
-	if (gethostname(buf, buflen) != 0)
-		return NULL;
-	buf[buflen - 1] = '\0';
-	if (strcmp(buf, "(none)") == 0 ||
-	    strcmp(buf, "localhost") == 0 ||
-	    strncmp(buf, "localhost.", strlen("localhost.")) == 0 ||
-	    buf[0] == '.')
-		return NULL;
-
-	if (short_hostname) {
-		p = strchr(buf, '.');
-		if (p)
-			*p = '\0';
-	}
-
-	return buf;
-}
-
-/* Handy function to get the time.
- * We only care about time advancements, not the actual time itself
- * Which is why we use CLOCK_MONOTONIC, but it is not available on all
- * platforms.
- */
-#define NO_MONOTONIC "host does not support a monotonic clock - timing can skew"
-int
-get_monotonic(struct timespec *ts)
-{
-
-#if defined(_POSIX_MONOTONIC_CLOCK) && defined(CLOCK_MONOTONIC)
-	return clock_gettime(CLOCK_MONOTONIC, ts);
-#elif defined(__APPLE__)
-	/* We can use mach kernel functions here.
-	 * This is crap though - why can't they implement clock_gettime?*/
-	static struct mach_timebase_info info = { 0, 0 };
-	static double factor = 0.0;
-	uint64_t nano;
-	long rem;
-
-	if (!posix_clock_set) {
-		if (mach_timebase_info(&info) == KERN_SUCCESS) {
-			factor = (double)info.numer / (double)info.denom;
-			clock_monotonic = posix_clock_set = 1;
-		}
-	}
-	if (clock_monotonic) {
-		nano = mach_absolute_time();
-		if ((info.denom != 1 || info.numer != 1) && factor != 0.0)
-			nano *= factor;
-		ts->tv_sec = nano / NSEC_PER_SEC;
-		ts->tv_nsec = nano % NSEC_PER_SEC;
-		if (ts->tv_nsec < 0) {
-			ts->tv_sec--;
-			ts->tv_nsec += NSEC_PER_SEC;
-		}
-		return 0;
-	}
-#endif
-
-#if 0
-	/* Something above failed, so fall back to gettimeofday */
-	if (!posix_clock_set) {
-		logger(NULL, LOG_WARNING, NO_MONOTONIC);
-		posix_clock_set = 1;
-	}
-#endif
-	{
-		struct timeval tv;
-		if (gettimeofday(&tv, NULL) == 0) {
-			TIMEVAL_TO_TIMESPEC(&tv, ts);
-			return 0;
-		}
-	}
-
-	return -1;
-}
-
 #if USE_LOGFILE
 void
 logger_open(struct dhcpcd_ctx *ctx)
@@ -175,6 +89,21 @@ logger_close(struct dhcpcd_ctx *ctx)
 	closelog();
 }
 
+/*
+ * NetBSD's gcc has been modified to check for the non standard %m in printf
+ * like functions and warn noisily about it that they should be marked as
+ * syslog like instead.
+ * This is all well and good, but our logger also goes via vfprintf and
+ * when marked as a sysloglike funcion, gcc will then warn us that the
+ * function should be printflike instead!
+ * This creates an infinte loop of gcc warnings.
+ * Until NetBSD solves this issue, we have to disable a gcc diagnostic
+ * for our fully standards compliant code in the logger function.
+ */
+#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 5))
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wmissing-format-attribute"
+#endif
 void
 logger(struct dhcpcd_ctx *ctx, int pri, const char *fmt, ...)
 {
@@ -184,9 +113,6 @@ logger(struct dhcpcd_ctx *ctx, int pri, const char *fmt, ...)
 	char fmt_cpy[1024];
 #endif
 
-	if (pri >= LOG_DEBUG && ctx && !(ctx->options & DHCPCD_DEBUG))
-		return;
-
 	serrno = errno;
 	va_start(va, fmt);
 
@@ -207,8 +133,18 @@ logger(struct dhcpcd_ctx *ctx, int pri, const char *fmt, ...)
 				fmt_left -= 2;
 				p++;
 			} else if (p[0] == '%' && p[1] == 'm') {
-				if (serr == NULL)
+				if (serr == NULL) {
+					/* strerror_r isn't portable.
+					 * strerror_l isn't widely found
+					 * and also problematic to use.
+					 * Also, if strerror_l exists then
+					 * strerror could easily be made
+					 * treadsafe in the same libc.
+					 * dhcpcd is only threaded in RTEMS
+					 * where strerror is threadsafe,
+					 * so this should be fine. */
 					serr = strerror(serrno);
+				}
 				fmt_wrote = strlcpy(fp, serr, fmt_left);
 				if (fmt_wrote > fmt_left)
 					break;
@@ -225,85 +161,114 @@ logger(struct dhcpcd_ctx *ctx, int pri, const char *fmt, ...)
 		*fp++ = '\0';
 		fmt = fmt_cpy;
 	}
-
 #endif
 
-	if (ctx == NULL || !(ctx->options & DHCPCD_QUIET)) {
+	if ((ctx == NULL || !(ctx->options & DHCPCD_QUIET)) &&
+	    (pri < LOG_DEBUG || (ctx->options & DHCPCD_DEBUG)))
+	{
 		va_list vac;
 
 		va_copy(vac, va);
+#ifdef HAVE_PRINTF_M
+		errno = serrno;
+#endif
 		vfprintf(pri <= LOG_ERR ? stderr : stdout, fmt, vac);
 		fputc('\n', pri <= LOG_ERR ? stderr : stdout);
 		va_end(vac);
 	}
 
-#ifdef HAVE_PRINTF_M
-	errno = serrno;
-#endif
+	/* Don't send to syslog if dumping leases or testing */
+	if (ctx->options & (DHCPCD_DUMPLEASE | DHCPCD_TEST))
+		goto out;
+
 	if (ctx && ctx->log_fd != -1) {
-		struct timeval tv;
-		char buf[32];
+		if (pri < LOG_DEBUG || (ctx->options & DHCPCD_DEBUG)) {
+			struct timeval tv;
+			char buf[32];
 
-		/* Write the time, syslog style. month day time - */
-		if (gettimeofday(&tv, NULL) != -1) {
-			time_t now;
-			struct tm tmnow;
+			/* Write the time, syslog style. month day time - */
+			if (gettimeofday(&tv, NULL) != -1) {
+				time_t now;
+				struct tm tmnow;
 
-			tzset();
-			now = tv.tv_sec;
-			localtime_r(&now, &tmnow);
-			strftime(buf, sizeof(buf), "%b %d %T ", &tmnow);
-			dprintf(ctx->log_fd, "%s", buf);
+				tzset();
+				now = tv.tv_sec;
+				localtime_r(&now, &tmnow);
+				strftime(buf, sizeof(buf), "%b %d %T ", &tmnow);
+				dprintf(ctx->log_fd, "%s", buf);
+			}
+
+#ifdef HAVE_PRINTF_M
+			errno = serrno;
+#endif
+			vdprintf(ctx->log_fd, fmt, va);
+			dprintf(ctx->log_fd, "\n");
 		}
-
-		vdprintf(ctx->log_fd, fmt, va);
-		dprintf(ctx->log_fd, "\n");
-	} else
+	} else {
+#ifdef HAVE_PRINTF_M
+		errno = serrno;
+#endif
 		vsyslog(pri, fmt, va);
+	}
+out:
 	va_end(va);
 }
 #endif
+#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 5))
+#pragma GCC diagnostic pop
+#endif
 
 ssize_t
 setvar(struct dhcpcd_ctx *ctx,
-    char ***e, const char *prefix, const char *var, const char *value)
+    char **e, const char *prefix, const char *var, const char *value)
 {
 	size_t len = strlen(var) + strlen(value) + 3;
 
 	if (prefix)
 		len += strlen(prefix) + 1;
-	**e = malloc(len);
-	if (**e == NULL) {
+	*e = malloc(len);
+	if (*e == NULL) {
 		logger(ctx, LOG_ERR, "%s: %m", __func__);
 		return -1;
 	}
 	if (prefix)
-		snprintf(**e, len, "%s_%s=%s", prefix, var, value);
+		snprintf(*e, len, "%s_%s=%s", prefix, var, value);
 	else
-		snprintf(**e, len, "%s=%s", var, value);
-	(*e)++;
+		snprintf(*e, len, "%s=%s", var, value);
 	return (ssize_t)len;
 }
 
 ssize_t
 setvard(struct dhcpcd_ctx *ctx,
-    char ***e, const char *prefix, const char *var, size_t value)
+    char **e, const char *prefix, const char *var, size_t value)
 {
+
 	char buffer[32];
 
 	snprintf(buffer, sizeof(buffer), "%zu", value);
 	return setvar(ctx, e, prefix, var, buffer);
 }
 
-
-time_t
-uptime(void)
+ssize_t
+addvar(struct dhcpcd_ctx *ctx,
+    char ***e, const char *prefix, const char *var, const char *value)
 {
-	struct timespec tv;
+	ssize_t len;
 
-	if (get_monotonic(&tv) == -1)
-		return -1;
-	return tv.tv_sec;
+	len = setvar(ctx, *e, prefix, var, value);
+	if (len != -1)
+		(*e)++;
+	return (ssize_t)len;
+}
+
+ssize_t
+addvard(struct dhcpcd_ctx *ctx,
+    char ***e, const char *prefix, const char *var, size_t value)
+{
+	char buffer[32];
+
+	snprintf(buffer, sizeof(buffer), "%zu", value);
+	return addvar(ctx, e, prefix, var, buffer);
 }
 
 char *
