@@ -32,6 +32,10 @@
 #include <net/if.h>
 
 #include "config.h"
+#ifdef HAVE_SYS_QUEUE_H
+#include <sys/queue.h>
+#endif
+
 #include "defs.h"
 #include "control.h"
 #include "if-options.h"
@@ -46,16 +50,23 @@
 #define LINK_DOWN	-1
 
 #define IF_DATA_IPV4	0
-#define IF_DATA_DHCP	1
-#define IF_DATA_IPV6	2
-#define IF_DATA_IPV6ND	3
-#define IF_DATA_DHCP6	4
-#define IF_DATA_MAX	5
+#define IF_DATA_ARP	1
+#define IF_DATA_IPV4LL	2
+#define IF_DATA_DHCP	3
+#define IF_DATA_IPV6	4
+#define IF_DATA_IPV6ND	5
+#define IF_DATA_DHCP6	6
+#define IF_DATA_MAX	7
 
 /* If the interface does not support carrier status (ie PPP),
  * dhcpcd can poll it for the relevant flags periodically */
 #define IF_POLL_UP	100	/* milliseconds */
 
+#ifdef __QNX__
+/* QNX carries defines for, but does not actually support PF_LINK */
+#undef IFLR_ACTIVE
+#endif
+
 struct interface {
 	struct dhcpcd_ctx *ctx;
 	TAILQ_ENTRY(interface) next;
@@ -99,13 +110,21 @@ struct dhcpcd_ctx {
 	char **ifcv;	/* configured interfaces */
 	unsigned char *duid;
 	size_t duid_len;
-	int link_fd;
 	struct if_head *ifaces;
 
+	int pf_inet_fd;
+#if defined(INET6) && defined(BSD)
+	int pf_inet6_fd;
+#endif
+#ifdef IFLR_ACTIVE
+	int pf_link_fd;
+#endif
+	int link_fd;
+
 #ifdef USE_SIGNALS
 	sigset_t sigset;
 #endif
-	struct eloop_ctx *eloop;
+	struct eloop *eloop;
 
 	int control_fd;
 	int control_unpriv_fd;
@@ -117,6 +136,8 @@ struct dhcpcd_ctx {
 	struct dhcp_opt *vivso;
 	size_t vivso_len;
 
+	char *randomstate; /* original state */
+
 #ifdef INET
 	struct dhcp_opt *dhcp_opts;
 	size_t dhcp_opts_len;
@@ -135,6 +156,8 @@ struct dhcpcd_ctx {
 	unsigned char secret[SECRET_LEN];
 	size_t secret_len;
 
+	struct dhcp_opt *nd_opts;
+	size_t nd_opts_len;
 	struct dhcp_opt *dhcp6_opts;
 	size_t dhcp6_opts_len;
 	struct ipv6_ctx *ipv6;
@@ -152,16 +175,12 @@ struct dhcpcd_ctx {
 };
 
 #ifdef USE_SIGNALS
-struct dhcpcd_siginfo {
-	int signo;
-};
-
-extern const int dhcpcd_handlesigs[];
-void dhcpcd_handle_signal(void *);
+extern const int dhcpcd_signals[];
+extern const size_t dhcpcd_signals_len;
 #endif
 
-int dhcpcd_oneup(struct dhcpcd_ctx *);
-int dhcpcd_ipwaited(struct dhcpcd_ctx *);
+int dhcpcd_ifafwaiting(const struct interface *);
+int dhcpcd_afwaiting(const struct dhcpcd_ctx *);
 pid_t dhcpcd_daemonise(struct dhcpcd_ctx *);
 
 int dhcpcd_handleargs(struct dhcpcd_ctx *, struct fd_list *, int, char **);
