@@ -508,8 +508,6 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct rt_msghdr *rtm)
 int
 if_route(unsigned char cmd, const struct rt *rt)
 {
-	const struct dhcp_state *state;
-	const struct ipv4ll_state *istate;
 	union sockunion {
 		struct sockaddr sa;
 		struct sockaddr_in sin;
@@ -522,6 +520,7 @@ if_route(unsigned char cmd, const struct rt *rt)
 	} rtm;
 	char *bp = rtm.buffer;
 	size_t l;
+	struct in_addr src_addr;
 
 #define ADDSU {								      \
 		l = RT_ROUNDUP(su.sa.sa_len);				      \
@@ -536,47 +535,36 @@ if_route(unsigned char cmd, const struct rt *rt)
 		ADDSU;							      \
 	}
 
-	if (cmd != RTM_DELETE) {
-		state = D_CSTATE(rt->iface);
-		istate = IPV4LL_CSTATE(rt->iface);
-	} else {
-		/* appease GCC */
-		state = NULL;
-		istate = NULL;
-	}
 	memset(&rtm, 0, sizeof(rtm));
 	rtm.hdr.rtm_version = RTM_VERSION;
 	rtm.hdr.rtm_seq = 1;
 	rtm.hdr.rtm_type = cmd;
 	rtm.hdr.rtm_addrs = RTA_DST;
-	if (cmd == RTM_ADD || cmd == RTM_CHANGE)
-		rtm.hdr.rtm_addrs |= RTA_GATEWAY;
 	rtm.hdr.rtm_flags = RTF_UP;
+	rtm.hdr.rtm_pid = getpid();
 #ifdef RTF_PINNED
 	if (cmd != RTM_ADD)
 		rtm.hdr.rtm_flags |= RTF_PINNED;
 #endif
 
-	if (cmd != RTM_DELETE) {
-		rtm.hdr.rtm_addrs |= RTA_IFA | RTA_IFP;
-		/* None interface subnet routes are static. */
-		if ((rt->gate.s_addr != INADDR_ANY ||
-		    rt->net.s_addr != state->net.s_addr ||
-		    rt->dest.s_addr !=
-		    (state->addr.s_addr & state->net.s_addr)) &&
-		    (istate == NULL ||
-		    rt->dest.s_addr !=
-		    (istate->addr.s_addr & inaddr_llmask.s_addr) ||
-		    rt->net.s_addr != inaddr_llmask.s_addr))
-			rtm.hdr.rtm_flags |= RTF_STATIC;
-		else {
+	if (cmd == RTM_ADD || cmd == RTM_CHANGE) {
+		int subnet;
+
+		rtm.hdr.rtm_addrs |= RTA_GATEWAY | RTA_IFA | RTA_IFP;
+		/* Subnet routes are clonning or connected if supported.
+		 * All other routes are static. */
+		subnet = ipv4_srcaddr(rt, &src_addr);
+		if (subnet == 1) {
 #ifdef RTF_CLONING
 			rtm.hdr.rtm_flags |= RTF_CLONING;
 #endif
 #ifdef RTP_CONNECTED
 			rtm.hdr.rtm_priority = RTP_CONNECTED;
 #endif
-		}
+		} else
+			rtm.hdr.rtm_flags |= RTF_STATIC;
+		if (subnet == -1) /* unikely */
+			rtm.hdr.rtm_addrs &= ~RTA_IFA;
 	}
 	if (rt->net.s_addr == htonl(INADDR_BROADCAST) &&
 	    rt->gate.s_addr == htonl(INADDR_ANY))
@@ -604,19 +592,20 @@ if_route(unsigned char cmd, const struct rt *rt)
 		rtm.hdr.rtm_addrs |= RTA_NETMASK;
 		if (rtm.hdr.rtm_flags & RTF_STATIC)
 			rtm.hdr.rtm_flags |= RTF_GATEWAY;
+		if (rt->net.s_addr == htonl(INADDR_BROADCAST))
+			rtm.hdr.rtm_flags |= RTF_HOST;
 	}
 	if ((cmd == RTM_ADD || cmd == RTM_CHANGE) &&
 	    !(rtm.hdr.rtm_flags & RTF_GATEWAY))
-		rtm.hdr.rtm_addrs |= RTA_IFA | RTA_IFP;
+		rtm.hdr.rtm_addrs |= RTA_IFP;
 
 	ADDADDR(&rt->dest);
 	if (rtm.hdr.rtm_addrs & RTA_GATEWAY) {
-#ifdef RTF_CLONING
-		if ((rtm.hdr.rtm_flags & (RTF_HOST | RTF_CLONING) &&
-#else
 		if ((rtm.hdr.rtm_flags & RTF_HOST &&
+		    rt->gate.s_addr == htonl(INADDR_ANY)) ||
+#ifdef RTF_CLONING
+		    rtm.hdr.rtm_flags & RTF_CLONING ||
 #endif
-		    rt->gate.s_addr != htonl(INADDR_LOOPBACK)) ||
 		    !(rtm.hdr.rtm_flags & RTF_STATIC))
 		{
 			if_linkaddr(&su.sdl, rt->iface);
@@ -638,7 +627,7 @@ if_route(unsigned char cmd, const struct rt *rt)
 		}
 
 		if (rtm.hdr.rtm_addrs & RTA_IFA)
-			ADDADDR(istate == NULL ? &state->addr : &istate->addr);
+			ADDADDR(&src_addr);
 
 		if (rt->mtu) {
 			rtm.hdr.rtm_inits |= RTV_MTU;
@@ -933,6 +922,7 @@ if_route6(unsigned char cmd, const struct rt6 *rt)
 	rtm.hdr.rtm_seq = 1;
 	rtm.hdr.rtm_type = cmd;
 	rtm.hdr.rtm_flags = RTF_UP | (int)rt->flags;
+	rtm.hdr.rtm_pid = getpid();
 #ifdef RTF_PINNED
 	if (rtm.hdr.rtm_type != RTM_ADD)
 		rtm.hdr.rtm_flags |= RTF_PINNED;
@@ -1057,12 +1047,10 @@ if_addrflags6(const struct in6_addr *addr, const struct interface *ifp)
 int
 if_getlifetime6(struct ipv6_addr *ia)
 {
-	int r;
 	struct in6_ifreq ifr6;
 	time_t t;
 	struct in6_addrlifetime *lifetime;
 
-	r = -1;
 	memset(&ifr6, 0, sizeof(ifr6));
 	strlcpy(ifr6.ifr_name, ia->iface->name, sizeof(ifr6.ifr_name));
 	ifr6.ifr_addr.sin6_family = AF_INET6;
@@ -1485,24 +1473,6 @@ _if_checkipv6(int s, struct dhcpcd_ctx *ctx,
 		int override;
 #endif
 
-#ifdef ND6_IFF_IFDISABLED
-		if (del_if_nd6_flag(s, ifp, ND6_IFF_IFDISABLED) == -1) {
-			logger(ifp->ctx, LOG_ERR,
-			    "%s: del_if_nd6_flag: ND6_IFF_IFDISABLED: %m",
-			    ifp->name);
-			return -1;
-		}
-#endif
-
-#ifdef ND6_IFF_PERFORMNUD
-		if (set_if_nd6_flag(s, ifp, ND6_IFF_PERFORMNUD) == -1) {
-			logger(ifp->ctx, LOG_ERR,
-			    "%s: set_if_nd6_flag: ND6_IFF_PERFORMNUD: %m",
-			    ifp->name);
-			return -1;
-		}
-#endif
-
 #ifdef ND6_IFF_AUTO_LINKLOCAL
 		if (own) {
 			int all;
@@ -1531,18 +1501,11 @@ _if_checkipv6(int s, struct dhcpcd_ctx *ctx,
 		}
 #endif
 
-#ifdef SIOCIFAFATTACH
-		if (af_attach(s, ifp, AF_INET6) == -1) {
+#ifdef ND6_IFF_PERFORMNUD
+		if (set_if_nd6_flag(s, ifp, ND6_IFF_PERFORMNUD) == -1) {
 			logger(ifp->ctx, LOG_ERR,
-			    "%s: af_attach: %m", ifp->name);
-			return 1;
-		}
-#endif
-
-#ifdef SIOCGIFXFLAGS
-		if (set_ifxflags(s, ifp, own) == -1) {
-			logger(ifp->ctx, LOG_ERR,
-			    "%s: set_ifxflags: %m", ifp->name);
+			    "%s: set_if_nd6_flag: ND6_IFF_PERFORMNUD: %m",
+			    ifp->name);
 			return -1;
 		}
 #endif
@@ -1586,6 +1549,38 @@ _if_checkipv6(int s, struct dhcpcd_ctx *ctx,
 		} else if (ra == 0 && !own)
 			logger(ifp->ctx, LOG_WARNING,
 			    "%s: IPv6 kernel autoconf disabled", ifp->name);
+#endif
+
+		/* Enabling IPv6 by whatever means must be the
+		 * last action undertaken to ensure kernel RS and
+		 * LLADDR auto configuration are disabled where applicable. */
+
+#ifdef SIOCIFAFATTACH
+		if (af_attach(s, ifp, AF_INET6) == -1) {
+			logger(ifp->ctx, LOG_ERR,
+			    "%s: af_attach: %m", ifp->name);
+			return 1;
+		}
+#endif
+
+#ifdef SIOCGIFXFLAGS
+		if (set_ifxflags(s, ifp, own) == -1) {
+			logger(ifp->ctx, LOG_ERR,
+			    "%s: set_ifxflags: %m", ifp->name);
+			return -1;
+		}
+#endif
+
+#ifdef ND6_IFF_IFDISABLED
+		if (del_if_nd6_flag(s, ifp, ND6_IFF_IFDISABLED) == -1) {
+			logger(ifp->ctx, LOG_ERR,
+			    "%s: del_if_nd6_flag: ND6_IFF_IFDISABLED: %m",
+			    ifp->name);
+			return -1;
+		}
+#endif
+
+#ifdef ND6_IFF_ACCEPT_RTADV
 #ifdef ND6_IFF_OVERRIDE_RTADV
 		if (override == 0 && ra)
 			return ctx->ra_global;
@@ -1601,12 +1596,12 @@ _if_checkipv6(int s, struct dhcpcd_ctx *ctx,
 	if (ra == -1)
 		/* The sysctl probably doesn't exist, but this isn't an
 		 * error as such so just log it and continue */
-		logger(ifp->ctx, errno == ENOENT ? LOG_DEBUG : LOG_WARNING,
+		logger(ctx, errno == ENOENT ? LOG_DEBUG : LOG_WARNING,
 		    "IPV6CTL_ACCEPT_RTADV: %m");
 	else if (ra != 0 && own) {
-		logger(ifp->ctx, LOG_DEBUG, "disabling Kernel IPv6 RA support");
+		logger(ctx, LOG_DEBUG, "disabling Kernel IPv6 RA support");
 		if (set_inet6_sysctl(IPV6CTL_ACCEPT_RTADV, 0) == -1) {
-			logger(ifp->ctx, LOG_ERR, "IPV6CTL_ACCEPT_RTADV: %m");
+			logger(ctx, LOG_ERR, "IPV6CTL_ACCEPT_RTADV: %m");
 			return ra;
 		}
 		ra = 0;
