@@ -46,24 +46,6 @@
 #include <netinet/in.h>
 #include <net/route.h>
 
-/* Support older kernels */
-#ifndef IFLA_WIRELESS
-# define IFLA_WIRELESS (IFLA_MASTER + 1)
-#endif
-
-/* Linux has these in an enum and there is just no way to work
- * out of they exist at compile time. Silly silly silly. */
-#define IFLA_AF_SPEC			26
-#define IFLA_INET6_ADDR_GEN_MODE	8
-#define IN6_ADDR_GEN_MODE_NONE		1
-
-/* For some reason, glibc doesn't include newer flags from linux/if.h
- * However, we cannot include linux/if.h directly as it conflicts
- * with the glibc version. D'oh! */
-#ifndef IFF_LOWER_UP
-#define IFF_LOWER_UP	0x10000		/* driver signals L1 up		*/
-#endif
-
 #include <errno.h>
 #include <fcntl.h>
 #include <ctype.h>
@@ -90,6 +72,18 @@
 #endif
 int if_getssid_wext(const char *ifname, uint8_t *ssid);
 
+/* Support older kernels */
+#ifndef IFLA_WIRELESS
+#define IFLA_WIRELESS (IFLA_MASTER + 1)
+#endif
+
+/* For some reason, glibc doesn't include newer flags from linux/if.h
+ * However, we cannot include linux/if.h directly as it conflicts
+ * with the glibc version. D'oh! */
+#ifndef IFF_LOWER_UP
+#define IFF_LOWER_UP	0x10000		/* driver signals L1 up		*/
+#endif
+
 #define bpf_insn		sock_filter
 #define BPF_SKIPTYPE
 #define BPF_ETHCOOK		-ETH_HLEN
@@ -1335,9 +1329,8 @@ int
 if_route(unsigned char cmd, const struct rt *rt)
 {
 	struct nlmr nlm;
-	int retval = 0;
-	const struct dhcp_state *state;
-	const struct ipv4ll_state *istate;
+	struct in_addr src_addr;
+	int subnet;
 
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
@@ -1358,18 +1351,12 @@ if_route(unsigned char cmd, const struct rt *rt)
 	nlm.rt.rtm_family = AF_INET;
 	nlm.rt.rtm_table = RT_TABLE_MAIN;
 
-	state = D_CSTATE(rt->iface);
-	istate = IPV4LL_CSTATE(rt->iface);
-	if (cmd == RTM_DELETE)
+	if (cmd == RTM_DELETE) {
 		nlm.rt.rtm_scope = RT_SCOPE_NOWHERE;
-	else {
-		/* We only change route metrics for kernel routes */
-		if ((rt->dest.s_addr ==
-		    (state->addr.s_addr & state->net.s_addr) &&
-		    rt->net.s_addr == state->net.s_addr) ||
-		    (istate && rt->dest.s_addr ==
-		    (istate->addr.s_addr & inaddr_llmask.s_addr) &&
-		    rt->net.s_addr == inaddr_llmask.s_addr))
+		subnet = -1;
+	} else {
+		/* Subnet routes are RTPROT_KERNEL otherwise RTPROT_BOOT */
+		if ((subnet = ipv4_srcaddr(rt, &src_addr)) == 1)
 			nlm.rt.rtm_protocol = RTPROT_KERNEL;
 		else
 			nlm.rt.rtm_protocol = RTPROT_BOOT;
@@ -1387,37 +1374,36 @@ if_route(unsigned char cmd, const struct rt *rt)
 	nlm.rt.rtm_dst_len = inet_ntocidr(rt->net);
 	add_attr_l(&nlm.hdr, sizeof(nlm), RTA_DST,
 	    &rt->dest.s_addr, sizeof(rt->dest.s_addr));
-	if (nlm.rt.rtm_protocol == RTPROT_KERNEL) {
-		add_attr_l(&nlm.hdr, sizeof(nlm), RTA_PREFSRC,
-		    istate == NULL ? &state->addr.s_addr : &istate->addr.s_addr,
-		    sizeof(state->addr.s_addr));
+	if (cmd == RTM_ADD || cmd == RTM_CHANGE) {
+		if (rt->gate.s_addr != htonl(INADDR_ANY))
+			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_GATEWAY,
+			    &rt->gate.s_addr, sizeof(rt->gate.s_addr));
+		if (rt->gate.s_addr != htonl(INADDR_LOOPBACK))
+			add_attr_32(&nlm.hdr, sizeof(nlm), RTA_OIF,
+			    rt->iface->index);
+		if (subnet != -1) {
+			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_PREFSRC,
+			    &src_addr.s_addr, sizeof(src_addr.s_addr));
+		}
+		if (rt->mtu) {
+			char metricsbuf[32];
+			struct rtattr *metrics = (void *)metricsbuf;
+
+			metrics->rta_type = RTA_METRICS;
+			metrics->rta_len = RTA_LENGTH(0);
+			rta_add_attr_32(metrics, sizeof(metricsbuf),
+			    RTAX_MTU, rt->mtu);
+			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_METRICS,
+			    RTA_DATA(metrics),
+			    (unsigned short)RTA_PAYLOAD(metrics));
+		}
 	}
-	/* If a host route then don't add the gateway */
-	if ((cmd == RTM_ADD || cmd == RTM_CHANGE) &&
-	    rt->net.s_addr != INADDR_BROADCAST)
-		add_attr_l(&nlm.hdr, sizeof(nlm), RTA_GATEWAY,
-		    &rt->gate.s_addr, sizeof(rt->gate.s_addr));
 
-	if (rt->gate.s_addr != htonl(INADDR_LOOPBACK))
-		add_attr_32(&nlm.hdr, sizeof(nlm), RTA_OIF, rt->iface->index);
 	if (rt->metric)
 		add_attr_32(&nlm.hdr, sizeof(nlm), RTA_PRIORITY, rt->metric);
 
-	if (cmd != RTM_DELETE && rt->mtu) {
-		char metricsbuf[32];
-		struct rtattr *metrics = (void *)metricsbuf;
-
-		metrics->rta_type = RTA_METRICS;
-		metrics->rta_len = RTA_LENGTH(0);
-		rta_add_attr_32(metrics, sizeof(metricsbuf), RTAX_MTU, rt->mtu);
-		add_attr_l(&nlm.hdr, sizeof(nlm), RTA_METRICS,
-		    RTA_DATA(metrics), (unsigned short)RTA_PAYLOAD(metrics));
-	}
-
-	if (send_netlink(rt->iface->ctx, NULL,
-	    NETLINK_ROUTE, &nlm.hdr, NULL) == -1)
-		retval = -1;
-	return retval;
+	return send_netlink(rt->iface->ctx, NULL,
+	    NETLINK_ROUTE, &nlm.hdr, NULL);
 }
 
 static int
@@ -1463,11 +1449,10 @@ if_addrflags(__unused const struct in_addr *addr,
 
 #ifdef INET6
 int
-if_address6(const struct ipv6_addr *ap, int action)
+if_address6(const struct ipv6_addr *ia, int action)
 {
 	struct nlma nlm;
 	struct ifa_cacheinfo cinfo;
-	int retval = 0;
 /* IFA_FLAGS is not a define, but is was added at the same time
  * IFA_F_NOPREFIXROUTE was do use that. */
 #if defined(IFA_F_NOPREFIXROUTE) || defined(IFA_F_MANAGETEMPADDR)
@@ -1482,39 +1467,40 @@ if_address6(const struct ipv6_addr *ap, int action)
 		nlm.hdr.nlmsg_type = RTM_NEWADDR;
 	} else
 		nlm.hdr.nlmsg_type = RTM_DELADDR;
-	nlm.ifa.ifa_index = ap->iface->index;
+	nlm.ifa.ifa_index = ia->iface->index;
 	nlm.ifa.ifa_family = AF_INET6;
-	if (ap->addr_flags & IFA_F_TEMPORARY) {
+#ifdef IPV6_MANAGETEMPADDR
+	if (ia->flags & IPV6_AF_TEMPORARY) {
+		/* Currently the kernel filters out these flags */
 #ifdef IFA_F_NOPREFIXROUTE
 		flags |= IFA_F_TEMPORARY;
 #else
 		nlm.ifa.ifa_flags |= IFA_F_TEMPORARY;
 #endif
 	}
-#ifdef IFA_F_MANAGETEMPADDR
-	else if (ap->flags & IPV6_AF_AUTOCONF &&
-	    ip6_use_tempaddr(ap->iface->name))
+#elif IFA_F_MANAGETEMPADDR
+	if (ia->flags & IPV6_AF_AUTOCONF)
 		flags |= IFA_F_MANAGETEMPADDR;
 #endif
 
 	/* Add as /128 if no IFA_F_NOPREFIXROUTE ? */
-	nlm.ifa.ifa_prefixlen = ap->prefix_len;
+	nlm.ifa.ifa_prefixlen = ia->prefix_len;
 	/* This creates the aliased interface */
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LABEL,
-	    ap->iface->alias, (unsigned short)(strlen(ap->iface->alias) + 1));
+	    ia->iface->alias, (unsigned short)(strlen(ia->iface->alias) + 1));
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LOCAL,
-	    &ap->addr.s6_addr, sizeof(ap->addr.s6_addr));
+	    &ia->addr.s6_addr, sizeof(ia->addr.s6_addr));
 
 	if (action >= 0) {
 		memset(&cinfo, 0, sizeof(cinfo));
-		cinfo.ifa_prefered = ap->prefix_pltime;
-		cinfo.ifa_valid = ap->prefix_vltime;
+		cinfo.ifa_prefered = ia->prefix_pltime;
+		cinfo.ifa_valid = ia->prefix_vltime;
 		add_attr_l(&nlm.hdr, sizeof(nlm), IFA_CACHEINFO,
 		    &cinfo, sizeof(cinfo));
 	}
 
 #ifdef IFA_F_NOPREFIXROUTE
-	if (!IN6_IS_ADDR_LINKLOCAL(&ap->addr))
+	if (!IN6_IS_ADDR_LINKLOCAL(&ia->addr))
 		flags |= IFA_F_NOPREFIXROUTE;
 #endif
 #if defined(IFA_F_NOPREFIXROUTE) || defined(IFA_F_MANAGETEMPADDR)
@@ -1522,17 +1508,14 @@ if_address6(const struct ipv6_addr *ap, int action)
 		add_attr_32(&nlm.hdr, sizeof(nlm), IFA_FLAGS, flags);
 #endif
 
-	if (send_netlink(ap->iface->ctx, NULL, NETLINK_ROUTE, &nlm.hdr,
-	    NULL) == -1)
-		retval = -1;
-	return retval;
+	return send_netlink(ia->iface->ctx, NULL,
+	    NETLINK_ROUTE, &nlm.hdr, NULL);
 }
 
 int
 if_route6(unsigned char cmd, const struct rt6 *rt)
 {
 	struct nlmr nlm;
-	int retval = 0;
 
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
@@ -1595,10 +1578,8 @@ if_route6(unsigned char cmd, const struct rt6 *rt)
 		    RTA_DATA(metrics), (unsigned short)RTA_PAYLOAD(metrics));
 	}
 
-	if (send_netlink(rt->iface->ctx, NULL,
-	    NETLINK_ROUTE, &nlm.hdr, NULL) == -1)
-		retval = -1;
-	return retval;
+	return send_netlink(rt->iface->ctx, NULL,
+	    NETLINK_ROUTE, &nlm.hdr, NULL);
 }
 
 static int
@@ -1713,8 +1694,9 @@ add_attr_nest_end(struct nlmsghdr *n, struct rtattr *nest)
 }
 
 static int
-if_disable_autolinklocal(struct dhcpcd_ctx *ctx, int ifindex)
+if_disable_autolinklocal(struct dhcpcd_ctx *ctx, unsigned int ifindex)
 {
+#ifdef HAVE_IN6_ADDR_GEN_MODE_NONE
 	struct nlml nlm;
 	struct rtattr *afs, *afs6;
 
@@ -1723,7 +1705,7 @@ if_disable_autolinklocal(struct dhcpcd_ctx *ctx, int ifindex)
 	nlm.hdr.nlmsg_type = RTM_NEWLINK;
 	nlm.hdr.nlmsg_flags = NLM_F_REQUEST;
 	nlm.i.ifi_family = AF_INET6;
-	nlm.i.ifi_index = ifindex;
+	nlm.i.ifi_index = (int)ifindex;
 	afs = add_attr_nest(&nlm.hdr, sizeof(nlm), IFLA_AF_SPEC);
 	afs6 = add_attr_nest(&nlm.hdr, sizeof(nlm), AF_INET6);
 	add_attr_8(&nlm.hdr, sizeof(nlm), IFLA_INET6_ADDR_GEN_MODE,
@@ -1732,6 +1714,12 @@ if_disable_autolinklocal(struct dhcpcd_ctx *ctx, int ifindex)
 	add_attr_nest_end(&nlm.hdr, afs);
 
 	return send_netlink(ctx, NULL, NETLINK_ROUTE, &nlm.hdr, NULL);
+#else
+	UNUSED(ctx);
+	UNUSED(ifindex);
+	errno = ENOTSUP;
+	return -1;
+#endif
 }
 
 static const char *prefix = "/proc/sys/net/ipv6/conf";
@@ -1746,7 +1734,7 @@ if_checkipv6(struct dhcpcd_ctx *ctx, const struct interface *ifp, int own)
 	if (ifp == NULL)
 		ifname = "all";
 	else if (own) {
-		if (if_disable_autolinklocal(ctx, (int)ifp->index) == -1)
+		if (if_disable_autolinklocal(ctx, ifp->index) == -1)
 			logger(ctx, LOG_DEBUG,
 			    "%s: if_disable_autolinklocal: %m", ifp->name);
 	}
